﻿---------------------------------------------------------------------------------------------------
-- Debug Frame Script
---------------------------------------------------------------------------------------------------

-- Addon Initialize
IGAS:NewAddon("Cube.Mdl_Cube_Debug")

_G = _G
format = string.format

index = 0
selectNode = nil

-- Addon	
_Addon.OnSlashCmd = _Addon.OnSlashCmd + function(self, option)		
	if option and strupper(option) == "DEBUG" then
		Cube_Debug.Visible = true
		return true
	end
end

function tabF:OnSizeChanged()
	local container
	
	for i = 1, self.TabNum do
		container = self:GetTabByIndex(i).Container
		container.Height = self.Height - 40
		container.View.Height = container.Heigh
		container.View.Width = self.Width - 2
	end
end

function txtIn:OnEnterPressed()
	local func, rs, text
	local view, node

	if txtIn.Text ~= "" then
		func = loadstring("return "..txtIn.Text)
		if func then
			func, rs = pcall(func)			
		end
	end

	txtIn:ClearFocus()
	
	return newView(txtIn.Text, rs)
end

function timer:OnTimer()
	self.Parent.Message = ""
	self.Interval = 0
end

function Cube_Debug:OnHide()
	Menu.Visible = false
end

function Menu:OnShow()
	local node = self.Owner
	
	if node.Level ~= 1 and type(node.MetaData.Value) == "table" then
		-- this is not first node
		mnuShow:Enable()
		mnuDel:Enable()
	else
		mnuShow:Disable()
		mnuDel:Disable()
	end
	
	mnuDel.Enabled = (node.Level ~= 1)
	
	if getmetatable(node.MetaData.Value) then
		mnuShowMeta:Enable()
	else
		mnuShowMeta:Disable()
	end
	if type(node.MetaData.Key) == "table" then
		mnuShowKey:Enable()
	else
		mnuShowKey:Disable()
	end
end

function mnuShow:OnClick()
	local node = Menu.Owner
	
	return newView(GetTitle(node), node.MetaData.Value)
end

function mnuShowMeta:OnClick()
	local node = Menu.Owner
	local text = GetTitle(node)
	local value = node.MetaData.Value
	
	if type(getmetatable(value)) == "table" then
		value = getmetatable(value)
		text = "getmetatable("..text..")"
		
		return newView(text, value)
	end
end

function mnuShowKey:OnClick()
	local node = Menu.Owner
	local value = node.MetaData.Key
	local text = tostring(value)
	
	return newView(text, value)
end

_DelNode = nil
function mnuDel:OnClick()
	_DelNode = Menu.Owner
	return IGAS:MsgBox(L["Are you sure to delete this key?"], "c", DelNode)
end

_ModNode = nil
_ModValue = nil
function btnCommit:OnClick()	
	if selectNode and btnCommit.Text == L["Resume"] then
		local v = selectNode.MetaData.Value
		
		if type(v) ~= "thread" then return end
		
		coroutine.resume(v)		
		btnCommit.Visible = (coroutine.status(v) == "suspended")
		
		return
	end
	
	local value = GetModValue()
	
	_ModNode = selectNode
	
	if value ~= selectNode.MetaData.Value then
		_ModValue = value
		return IGAS:MsgBox(L["Are you sure to change the value?"], "c", ModNode)
	end
end

function code:OnClick()
	if Cube_Debug.Parent:GetChild("Cube_Main") then
		Cube_Debug.Parent:GetChild("Cube_Main").Visible = true
	end
end

function bugList:OnClick()
	if Cube_Debug.Parent:GetChild("Cube_BugList") then
		Cube_Debug.Parent:GetChild("Cube_BugList").Visible = true
	end
end

-- Scripts
function DelNode(flag)
	if flag and _DelNode then
		_DelNode.Parent.MetaData.Value[_DelNode.MetaData.Key] = nil
		_DelNode:Dispose()
	end
	_DelNode = nil
end

function ModNode(flag)
	if _ModNode then
		if flag then
			if _ModValue ~= nil then
				_ModNode.MetaData.Value = _ModValue
				_ModNode.Parent.MetaData.Value[_ModNode.MetaData.Key] = _ModValue
			else
				_ModNode.Parent.MetaData.Value[_ModNode.MetaData.Key] = nil
				_ModNode:Dispose()
				_ModNode = nil
				return
			end
		end
		UpdateChild(_ModNode)
		_ModNode = nil
		_ModValue = nil
	end
end

function Format(i, v)
	local value = ""

	if v and type(v) == "userdata" then
		return tostring(i), tostring(i)
	end

	if i == nil then
		return "", ""
	elseif type(i) == "string" then
		value = string.format("%q", i)
	else
		value = tostring(i)
	end

	if v and type(v) == "table" and getmetatable(v) then
		return "\124cff1eff00[M]\124h\124r"..value, value
	else
		return value, value
	end
end

function GetTitle(node)
	local text = ""
	
	if not node then
		return text
	end
	
	while node.Level > 1 do
		if type(node.MetaData.Key) == "string" then
			text = ("[\"%s\"]"):format(node.MetaData.Key)..text
		else
			text = ("[%s]"):format(tostring(node.MetaData.Key))..text
		end
		
		node = node.Parent
	end
	
	text = node.MetaData.Key..text
	
	return text
end

function GetModValue()
	local code = "return "..(txtValue.Text or "")

	local func, err
	local t

	func, err = loadstring(code)
	if func then
		return func()
	end
end

function UpdateType(node)
	local v = node and node.MetaData.Value

	selectNode = node

	txtValue:ClearFocus()
	txtValue.Text = ""
	txtIn.Text = GetTitle(node)	
	
	btnCommit.Text = L["Commit"]

	if type(v) == "string" then
		txtValue.Visible = true
		txtValue.MouseEnabled = true
		btnCommit.Visible = true
		txtValue.Text = format("%q", v)
	elseif type(v) == "table" or type(v) == "function" or type(v) == "userdata" then
		txtValue.Visible = true
		txtValue.MouseEnabled = false
		btnCommit.Visible = false
		if v == _G then
			txtValue.Text = "_G"
		else
			txtValue.Text = tostring(v)
		end
	elseif type(v) == "number" or type(v) == "boolean" then
		txtValue.Visible = true
		txtValue.MouseEnabled = true
		btnCommit.Visible = true
		txtValue.Text = tostring(v)
	elseif type(v) == "thread" then
		txtValue.Visible = true
		txtValue.MouseEnabled = false
		btnCommit.Visible = (coroutine.status(v) == "suspended")
		btnCommit.Text = L["Resume"]
		txtValue.Text = tostring(v)
	end
end

function UpdateChild(node)
	local chNode, text, value, isThread
	
	if node and type(node.MetaData.Value) == "table" and (not node.CanToggle) and next(node.MetaData.Value) and node.MetaData.Value ~= _G then
		for i, v in pairsByKeys(node.MetaData.Value) do
			chNode = nil
			
			if type(i) == "thread" then
				isThread = true
			else
				isThread = false
			end
			
			text = tostring(i)
			
			if getmetatable(v) then
				chNode = node:AddNode{Text = "\124cff1eff00[M]\124h\124r"..text, Value = v, Key = i}
			else
				chNode = node:AddNode{Text = text, Value = v, Key = i}
			end
			
			if chNode then
				if isThread then
					chNode.FunctionName = L["Resume"]
				end
				chNode.Menu = Menu
			end
		end
	end
end

function OnNodeSelected(self, node)
	return UpdateType(node)
end

function OnNodeToggle(self, node)
	if node.ToggleState then
		-- Update Child
		for i = 1, node.ChildNodeCount do
			UpdateChild(node:GetNode(i))
		end
	end
end

function OnNodeFunctionClick(self, func, node)
	if node and func == L["Resume"] then
		if type(node.MetaData.Key) == "thread" then
			coroutine.resume(node.MetaData.Key)
			if coroutine.status(node.MetaData.Key) ~= "suspended" then
				node.FunctionName = nil
			end
		end
	end
end

function newView(text, value)
	if type(value) ~= "table" then
		Cube_Debug.Message = L["Value is not table."]
		timer.Interval = 2
		return
	end
	
	if value == _G then
		Cube_Debug.Message = L["Not support for _G."]
		timer.Interval = 2
		return
	end

	index = index + 1

	local tab = tabF:AddTab("Watch"..index)
	local container = tab.Container
	local view = TreeView("View", container)
	local node
	
	view:SetPoint("TOPLEFT", container)
	container.Height = tabF.Height - 40
	view.Height = container.Height
	view.Width = tabF.Width - 2

	view.OnNodeSelected = OnNodeSelected
	view.OnNodeToggle = OnNodeToggle
	view.OnNodeFunctionClick = OnNodeFunctionClick

	tab.Selected = true	
	
	if getmetatable(value) then
		node = view:AddNode{Text = "\124cff1eff00[M]\124h\124r"..text, Value = value, Key = text}
	else
		node = view:AddNode{Text = text, Value = value, Key = text}
	end
	node.Menu = Menu
	UpdateChild(node)
	node:Select()	

	return view
end
