-- Author      : Kurapica
-- Change Log  :
--				2011/03/13	Recode as class

---------------------------------------------------------------------------------------------------------------------------------------
--- CheckBox is a widget type using for boolean selection with a label.
-- <br><br>inherit <a href="..\Base\Button.html">Button</a> For all methods, properties and scriptTypes
-- @name CheckBox
-- @class table
-- @field Checked whether the checkbox is checked
-- @field Text the text to be displayed for informations
-- @filed TrueText the text to be displayed when the checkbox is checked
-- @field FalseText the text to be displyed when the checkbox is un-checked
---------------------------------------------------------------------------------------------------------------------------------------

-- Check Version
local version = 8

if not IGAS:NewAddon("IGAS.Widget.CheckBox", version) then
	return
end

class "CheckBox"
	inherit "Button"

	-- Script
    local function OnClick(self, ...)
		if self.InDesignMode then return end
        self:GetChild("ChkBtn").Checked = not self:GetChild("ChkBtn").Checked
		if self:GetChild("ChkBtn").Checked and self.TrueText and self.TrueText ~= "" then
			self.Text = self.TrueText
		elseif (not self:GetChild("ChkBtn").Checked) and self.FalseText and self.FalseText ~= "" then
			self.Text = self.FalseText
		end
        return self:Fire("OnValueChanged", self:GetChild("ChkBtn").Checked)
    end

    local function OnEnter(self)
        self:GetHighlightTexture():Show()
    end

    local function OnLeave(self)
        self:GetHighlightTexture():Hide()
    end

	------------------------------------------------------
	-- Script
	------------------------------------------------------
	------------------------------------
	--- ScriptType, Run when the checkbox's checking state is changed
	-- @name CheckBox:OnValueChanged
	-- @class function
	-- @param checked true if the checkbox is checked
	-- @usage function CheckBox:OnValueChanged(checked)<br>
	--    -- do someting<br>
	-- end
	------------------------------------
	script "OnValueChanged"

	------------------------------------------------------
	-- Method
	------------------------------------------------------	
	------------------------------------
	--- Sets the checkbox's checking state
	-- @name CheckBox:SetChecked
	-- @class function
	-- @param checked true if set the checkbox to be checked
	-- @usage CheckBox:SetChecked(true)
	------------------------------------
	function SetChecked(self, flag)
		self:GetChild("ChkBtn").Checked = (flag and true) or false
		if self:GetChild("ChkBtn").Checked and self.TrueText and self.TrueText ~= "" then
			self.Text = self.TrueText
		elseif (not self:GetChild("ChkBtn").Checked) and self.FalseText and self.FalseText ~= "" then
			self.Text = self.FalseText
		end
	end

	------------------------------------
	--- Gets the checkbox's checking state
	-- @name CheckBox:GetChecked
	-- @class function
	-- @return checked - true if set the checkbox to be checked
	-- @usage CheckBox:GetChecked()
	------------------------------------
	function GetChecked(self)
		return self:GetChild("ChkBtn").Checked or false
	end

	------------------------------------
	--- Sets the checkbox's label
	-- @name CheckBox:SetText
	-- @class function
	-- @param text the text to be displyed
	-- @usage CheckBox:SetText("Enable the addon")
	------------------------------------
	function SetText(self, text)
		self:GetChild("Text").Text = text
	end

	------------------------------------
	--- Gets the checkbox's label
	-- @name CheckBox:GetText
	-- @class function
	-- @return text - the text to be displyed
	-- @usage CheckBox:GetText()
	------------------------------------
	function GetText(self)
		return self:GetChild("Text").Text
	end

	------------------------------------
	--- Enable the checkbox, make it clickable
	-- @name CheckBox:Enable
	-- @class function
	-- @usage CheckBox:Enable()
	------------------------------------
	function Enable(self)
		self.__UI:Enable()
		self:GetChild("ChkBtn"):Enable()
	end

	------------------------------------
	--- Disable the checkbox, make it un-clickable
	-- @name CheckBox:Disable
	-- @class function
	-- @usage CheckBox:Disable()
	------------------------------------
	function Disable(self)
		self.__UI:Disable()
		self:GetChild("ChkBtn"):Disable()
	end

	------------------------------------------------------
	-- Property
	------------------------------------------------------
	-- Checked
	property "Checked" {
		Set = function(self, value)
			self:SetChecked(value)
		end,

		Get = function(self)
			return (self:GetChecked() and true) or false
		end,

		Type = Boolean,
	}
	-- Text
	property "Text" {
		Set = function(self, text)
			self:SetText(text)
			self.Width = self:GetChild("Text").Width + 26
		end,

		Get = function(self)
			return self:GetText()
		end,

		Type = String,
	}
	-- TrueText
	property "TrueText" {
		Set = function(self, text)
			self.__TrueText = tostring(text or "")
			
			if self:GetChild("ChkBtn").Checked and self.__TrueText and self.__TrueText ~= "" then
				self.Text = self.__TrueText
			elseif (not self:GetChild("ChkBtn").Checked) and self.__FalseText and self.__FalseText ~= "" then
				self.Text = self.__FalseText
			end
		end,

		Get = function(self)
			return self.__TrueText or ""
		end,

		Type = String,
	}
	-- FalseText
	property "FalseText" {
		Set = function(self, text)
			self.__FalseText = tostring(text or "")
			
			if self:GetChild("ChkBtn").Checked and self.__TrueText and self.__TrueText ~= "" then
				self.Text = self.__TrueText
			elseif (not self:GetChild("ChkBtn").Checked) and self.__FalseText and self.__FalseText ~= "" then
				self.Text = self.__FalseText
			end
		end,

		Get = function(self)
			return self.__FalseText or ""
		end,

		Type = String,
	}

	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
	function CheckBox(name, parent)
		-- New Frame
		local frame = Button(name,parent)

        frame.Height = 24
        frame.Width = 100
        frame.MouseEnabled = true

        local chkBtn = CheckButton("ChkBtn", frame)
        chkBtn:SetPoint("LEFT", frame, "LEFT")
        chkBtn.MouseEnabled = false

		chkBtn.Height = 26
		chkBtn.Width = 26

		chkBtn:SetNormalTexture([[Interface\Buttons\UI-CheckBox-Up]])
		chkBtn:SetPushedTexture([[Interface\Buttons\UI-CheckBox-Down]])
		chkBtn:SetCheckedTexture([[Interface\Buttons\UI-CheckBox-Check]])
		chkBtn:SetDisabledCheckedTexture([[Interface\Buttons\UI-CheckBox-Check-Disabled]])
		
        local text = FontString("Text",frame,"OVERLAY","GameFontNormal")
        text.JustifyH = "LEFT"
		text.JustifyV = "MIDDLE"
        text:SetPoint("TOP", frame, "TOP")
        text:SetPoint("LEFT", chkBtn, "RIGHT")
        text:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT")
        text.Text = ""

        frame:SetHighlightTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")
        local t = frame:GetHighlightTexture()
        t:SetBlendMode("ADD")
        t:SetAllPoints(text)

        frame.OnEnter = frame.OnEnter + OnEnter
        frame.OnLeave = frame.OnLeave + OnLeave
        frame.OnClick = frame.OnClick + OnClick

		return frame
	end
endclass "CheckBox"
